@@ -7,9 +7,10 @@ use crate::paint_tool::PaintOperation;
 use crate::paint_tool::PaintTool;
 use crate::piston_wrapper::kusa_image::KusaImage;
 use crate::settings::Settings;
+use crate::KusaApp;
 use piston_window::*;
 
-pub fn show_window(settings: Settings, k_image: &mut KusaImage) {
+pub fn show_window(app: &KusaApp, mut settings: Settings, k_image: &mut KusaImage) {
     let opengl = OpenGL::V3_2;
 
     let width = settings.canvas_margin_left
@@ -43,12 +44,15 @@ pub fn show_window(settings: Settings, k_image: &mut KusaImage) {
     let mut count_to_reload: u64 = 0;
     // Event loop.
     window.set_lazy(true);
+    // フレームではなく、イベントが起こると１つ進む（＾～＾）？
     while let Some(e) = window.next() {
-        if count_to_reload % 1000 == 999 {
+        //println!("Trace   | window.next() time={}", count_to_reload); // FPSが分からん（＾～＾）
+        if count_to_reload % 120 == 119 {
             // ミリ秒の取り方が分からなかったぜ☆（＾～＾）
             // イベント・ループの中で　ファイル入出力するのは　クソだが　使い慣れてないんで仕方ないぜ☆（＾～＾）
             // 設定ファイルを監視するぜ☆（＾～＾）
-            let settings = match Settings::load() {
+            println!("Debug   | Reload settings");
+            settings = match Settings::load(&app.settings_path) {
                 Ok(x) => x,
                 Err(why) => panic!("Settings load fail: {}", why),
             };
